<?php
namespace Financeiro\Logic\ContasPagar\Cnab\Remessa;

use \Geral\Util\Ambiente;
use DateTime;
use Financeiro\Entities\Cnabpag;
use Financeiro\Logic\ContasPagar\BoletoPagar\BoletoPagar;
use Geral\Util\DateUtil;
use Geral\Util\TratamentoDeMensagens;
use Setforms2\Shared\Registry;
use Fiscal\Entities\Boletpagar;
use Financeiro\Logic\ContasPagar\Cnab\Remessa\GeraCnab;
use Indeterminado\Entities\Logboletorempag;

/**
 * Created by Setforms Builder
 */
class OperacoesCnab {

    const DIR_ARQUIVOS_CNAB_PAG = "/public/Cnab/Pagamento/";
    const PREFIXO_NOME_ARQUIVO = 'CNABPAG';

    /**
     * @param $nrorg
     * @param $cdempresa - Código da Empresa
     * @param $nrcnabpag - Número da Remessa
     * @param $idtipcnabpag - 0 Remessa 1 Retorno
     * @param \DateTime $dtcnabpag - Data de inclusão do Cnab
     * @param $cdoperador - Operador de Inclusão
     * @param $cdreducont - Código Reduzido da Conta
     * @param $cdbanco - Código do Banco
     * @param $idopecnabpag - I Inclusao E Exclusão B BLOQUEIO
     * @param $cdfilial - Filial
     * @param $vrtotalcnabpag -  Valor Total da Remessa
     * @param $qtbolcnabpag - Quantidade de Boletos
     * @param $nrseqremessapag - Sequencial de Remessa (Novo Coódigo de Acordo com Cdreducont)
     * @param $idsitucnabp - Situação CNAB (Ver Constantes em \Entities\BoletPagar)
     * @param array $boletos BoletPagar
     * @param bool $autorizaAgendamento - Banco Bradesco
     * @throws \Exception
     * @return Exception|\Financeiro\Logic\ContasPagar\Cnab\Remessa\GeraCnabSantander
     */
    public static function insereRemessa($nrorg, $cdempresa, $nrcnabpag, $idtipcnabpag, DateTime $dtcnabpag, $cdoperador,
                                         $cdreducont,$cdbanco, $idopecnabpag, $cdfilial, $vrtotalcnabpag, $qtbolcnabpag,
                                         $nrseqremessapag, $idsitucnabp, $boletos = array(),
                                         $autorizaAgendamento = false){
        try {

            $nrorg = Ambiente::getNrorg();
            $caminhoArquivo = APP_PATH.self::DIR_ARQUIVOS_CNAB_PAG.$nrorg."/";
            if (!is_dir($caminhoArquivo)) {
                mkdir($caminhoArquivo, "0700", true);
            }

                $caminhoArquivo .= self::PREFIXO_NOME_ARQUIVO.
                $cdempresa.$nrcnabpag;

            /** @var $remessa Cnabpag */
           $remessa = self::gravaRemessa($nrorg, $cdempresa, $nrcnabpag, $idtipcnabpag, $dtcnabpag, $cdoperador,
                $cdreducont, $cdbanco, $idopecnabpag, $cdfilial, $vrtotalcnabpag, $qtbolcnabpag,
                $nrseqremessapag, $caminhoArquivo);

            self::alteraBoletos($boletos, $nrcnabpag, $idsitucnabp);
            self::gravaArquivo($caminhoArquivo, $remessa, $boletos, $autorizaAgendamento);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $caminhoArquivo
     * @param $remessa
     * @param $boletos
     * @param bool $autorizaAgendamento
     */
    private function gravaArquivo($caminhoArquivo, $remessa, $boletos, $autorizaAgendamento =  false){
        try {
            $geraCnab = GeraCnab::factory($remessa);
            $geraCnab->geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabpag
     * @param $idtipcnabpag
     * @param DateTime $dtcnabpag
     * @param $cdoperador
     * @param $cdreducont
     * @param $cdbanco
     * @param $idopecnabpag
     * @param $cdfilial
     * @param $vrtotalcnabpag
     * @param $qtbolcnabpag
     * @param $nrseqremessapag
     * @param $caminhoArquivo
     * @throws \Exception
     * @return CnabPag Cnabpag
     */
    private function gravaRemessa($nrorg, $cdempresa, $nrcnabpag, $idtipcnabpag, DateTime $dtcnabpag , $cdoperador,
                                  $cdreducont, $cdbanco, $idopecnabpag, $cdfilial, $vrtotalcnabpag,$qtbolcnabpag,
                                  $nrseqremessapag, $caminhoArquivo)    {
        try
        {
            $cnabpag = new Cnabpag();
            $cnabpag->setNrorg($nrorg);
            $cnabpag->setCdempresa($cdempresa);
            $cnabpag->setNrcnabpag($nrcnabpag);
            $cnabpag->setDsarqcnabpag($caminhoArquivo);
            $cnabpag->setIdtipcnabpag($idtipcnabpag);
            $cnabpag->setDtcnabpag($dtcnabpag);
            $cnabpag->setCdoperador($cdoperador);
            $cnabpag->setCdreducont($cdreducont);
            $cnabpag->setCdbanco($cdbanco);
            $cnabpag->setIdopecnabpag($idopecnabpag);
            $cnabpag->setcdfilial($cdfilial);
            $cnabpag->setVrtotalcnabpag($vrtotalcnabpag);
            $cnabpag->setQtbolcnabpag($qtbolcnabpag);
            $cnabpag->setNrseqremessapag($nrseqremessapag);
            $cnabpag->setDtinclusao(DateUtil::getDataAtual());

            $cnabpag->persist();
            Registry::getEntityManager()->flush();

            return $cnabpag;
        }catch(\Exception $e)
        {
            throw $e;
        }
    }

    /**
     * @param array $boletos
     * @param $nrcnabpag
     * @param $idsitucnabp
     * @throws \Exception
     */
    private function alteraBoletos($boletos = Array(), $nrcnabpag, $idsitucnabp ){
        try {
            /** @var $boleto Boletpagar */
            foreach ($boletos as $boleto) {
                self::alteraBoleto($boleto, $nrcnabpag, $idsitucnabp);
            }
            Registry::getEntityManager()->flush();
        } catch (\Exception $e)
        {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @param $nrcnabpag
     * @param $idsitucnabp
     * @throws \Exception
     */
    private function alteraBoleto( Boletpagar $boleto, $nrcnabpag, $idsitucnabp ){
        try {
            /** @var $boleto BoletPagar */
            $boleto = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($boleto->getCdfilial(),
                                                                          $boleto->getNrlanbolpag(),
                                                                          $boleto->getNrorg());
            $boleto->setNrcnabpagrem($nrcnabpag);
            $boleto->setIdsitucnabp($idsitucnabp);
            $boleto->persist();

        } catch (\Exception $e)
        {
            throw $e;
        }
    }

    /**
     * @param $cdempresa
     * @param $nrcnabpag
     * @internal param $caminhoArquivo
     */
    public static function downloadArquivo($cdempresa,$nrcnabpag)
    {

        try{
            $nrorg = Ambiente::getNrorg();
            $caminhoArquivo = APP_PATH.self::DIR_ARQUIVOS_CNAB_PAG.$nrorg."/";
            $caminhoArquivo .= self::PREFIXO_NOME_ARQUIVO.$cdempresa.$nrcnabpag;
            $arquivo = new \TkFile($caminhoArquivo.'.txt');
            $arquivo->download();
        }
        catch(\Exception $e){
            TratamentoDeMensagens::exibirMensagens($e);


        }
    }

    /**
     * @param $cdempresa
     * @param array $boletos
     * @param $nrcnabpag
     * @throws \Exception
     * @internal param array $boleto
     */
    public static function cancelaBoletosRemessa($cdempresa, $boletos = array(), $nrcnabpag){

        try {
            /** @var $bol eto BoletPagar */
            foreach ($boletos as $boleto) {

                // So retira o boleto da remessa se o mesmo não estiver baixado
                if ($boleto->getIdsitucnabp() != Boletpagar::ID_SIT_CNAB_BAIXADO) {

                    //Insere o log do boleto
                    self::insereLogBoletoRem($boleto, $nrcnabpag, $cdempresa);

                    //Altera o valor e a quantidade de boletos na Remessa
                    self::subtraiValorRemessa($cdempresa, $nrcnabpag, $boleto->getVrfinalbpag());

                    $boleto->setNrcnabpagrem(null);
                    $boleto->setIdsitucnabp(Boletpagar::ID_SIT_CNAB_NAO_ENVIADO);
                    $boleto->persist();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @param $nrcnabpag
     * @param $cdempresa
     */
    private function insereLogBoletoRem(Boletpagar $boleto, $nrcnabpag, $cdempresa){

        $log = New Logboletorempag();
        $log->setNrorg(\Ambiente::getNrorg());
        $log->setCdfiliallogpag($boleto->getCdfilial());
        $log->setNrlanbolpaglog($boleto->getNrlanbolpag());
        $log->setCdopelogpag(\Ambiente::getCdOperador());
        $log->setNrcnabpagretlog($nrcnabpag);
        $log->setCdempresalogpag($cdempresa);
        $log->setDtinclusao(DateUtil::getDataAtual());
        $log->setDtlogboletrpag(DateUtil::getDataAtual());
        $log->persist();
    }

    /**
     * @param $cdempresa
     * @param $nrcnabpag
     * @param $valor
     */
    private function subtraiValorRemessa ($cdempresa, $nrcnabpag, $valor){
        /** @var $cnabpag Cnabpag  */
        $cnabpag = Cnabpag::findOneByNrcnabpagAndCdempresaAndNrorg($nrcnabpag, $cdempresa, \Ambiente::getNrorg());
        $cnabpag->setVrtotalcnabpag($cnabpag->getVrtotalcnabpag() - $valor);
        $cnabpag->setQtbolcnabpag($cnabpag->getQtbolcnabpag() - 1);
        $cnabpag->persist();
    }
}
